using System;
using System.Collections.Generic;
using System.Text;
using PernEdit.Plugin;
using System.Drawing;
using PernEdit.GraphicsHelpers;
using PernEdit.WorldEntities;
using PernEdit;

namespace PernEditor.Creators.Brushes
{
    /// <summary>
    /// A brush which stamps as metatiles
    /// </summary>
    public class BrushMetaTileStamper : IBrushStamperPlugin
    {

        /// <summary>
        /// Called when the drag operation iscomplete.
        /// </summary>
        /// <param name="x">The final x location.</param>
        /// <param name="y">The final y location.</param>
        public void Stamp(TileBrush brush, Layer layer, Rectangle area)
        {
            int x = area.X;
            int y = area.Y;

            x /= brush.BrushLayer.Width;
            x *= brush.BrushLayer.Width;

            y /= brush.BrushLayer.Height;
            y *= brush.BrushLayer.Height;

            int dx = area.Width;
            int dy = area.Height;

            Layer temp = new Layer("temp", dx, dy, brush.BrushLayer.TileWidth, brush.BrushLayer.TileHeight);

            for (int iy = 0; iy < dy; iy += brush.BrushLayer.Height)
            {
                for (int ix = 0; ix < dx; ix += brush.BrushLayer.Width)
                {
                    temp.Stamp(brush.BrushLayer, ix, iy, true);
                }
            }

            layer.Stamp(temp, area.X, area.Y, true);
        }

        /// <summary>
        /// Gets the bitmap.
        /// </summary>
        /// <param name="brush">The brush.</param>
        public Bitmap GetBitmap(TileBrush brush, Rectangle area)
        {
          return brush.BrushLayer.GetBitmap();
        }

        /// <summary>
        /// Enumerated type of the plugin
        /// </summary>
        /// <value></value>
        public PluginType Type
        {
            get { return PluginType.PLUGIN_TYPE_BRUSH; }
        }

        /// <summary>
        /// The name of the plugin.  Will be used by the app to list the plugin
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return "Metatile Stamper"; }
        }

        /// <summary>
        /// GUID unique identifier for the plugin to avoid name clashes
        /// </summary>
        /// <value></value>
        public Guid ID
        {
            get { return new Guid("3248E630-7B6C-4e5b-A99C-DCA03256AAC1"); }
        }

        /// <summary>
        /// Version of the Pernedit Library the library was built against
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return "1.0.0.0"; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return Name;
        }

        /// <summary>
        /// Gets the icon to be used in the tool bar.
        /// </summary>
        /// <value>The icon.</value>
        public System.Drawing.Bitmap Icon
        {
            get { return new Bitmap(Editor.GetPathFromRelative("Brushes\\MetaTileStamper\\stamp_basic.gif")); }
        }

        /// <summary>
        /// Gets the tool tip to be displayed in the tool bar
        /// </summary>
        /// <value></value>
        public string MenuToolTip
        {
            get { return "Stamps the brush alligned to a grid with spacing equal to the selected brush"; }
        }

    }
}
