using System;
using System.Collections.Generic;
using System.Text;
using PernEdit.Plugin;
using PernEdit.WorldEntities;
using PernEdit.GraphicsHelpers;
using System.Drawing;
using PernEdit;

namespace Default
{
    /// <summary>
    /// A random brush stamper
    /// </summary>
    public class BrushStamperRandom : IBrushStamperPlugin
    {
        private Random rand = new Random((int)DateTime.Now.Ticks);
        
        /// <summary>
        /// Called when the drag operation iscomplete.
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <param name="layer">The layer.</param>
        /// <param name="area">The area to be affected by the stamp.</param>
        public void Stamp(TileBrush brush, Layer layer, Rectangle area)
        {
            int dx = area.Width;
            int dy = area.Height;

            Layer temp = new Layer("temp", dx, dy, brush.BrushLayer.TileWidth, brush.BrushLayer.TileHeight);

            for (int iy = 0; iy < dy; iy += 1)
                for (int ix = 0; ix < dx; ix += 1)
                {
                    temp.Tiles[ix + iy * dx] = brush.BrushLayer.Tiles[rand.Next(brush.BrushLayer.Tiles.Length)];
                }

            layer.Stamp(temp, area.X, area.Y, true);

        }
        /// <summary>
        /// Gets the bitmap.
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <returns></returns>
        public Bitmap GetBitmap(TileBrush brush, Rectangle area)
        {
            
            int index = rand.Next(brush.BrushLayer.Height * brush.BrushLayer.Width);

            if (brush.BrushLayer.Tiles[index].index >= 0)
                return PernEdit.Editor.Instance.World.TileSets[brush.BrushLayer.Tiles[index].tileSetID].Tiles[brush.BrushLayer.Tiles[index].index];
            else
                return new Bitmap(brush.BrushLayer.TileWidth, brush.BrushLayer.TileHeight);
        }

        /// <summary>
        /// Enumerated type of the plugin
        /// </summary>
        /// <value></value>
        public PluginType Type
        {
            get { return PluginType.PLUGIN_TYPE_BRUSH; }
        }

        /// <summary>
        /// The name of the plugin.  Will be used by the app to list the plugin
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return "Random"; }
        }

        /// <summary>
        /// GUID unique identifier for the plugin to avoid name clashes
        /// </summary>
        /// <value></value>
        public Guid ID
        {
            get { return new Guid("37DF55E7-1430-4419-AA5C-981CEF8F06CD"); }
        }

        /// <summary>
        /// Version of the Pernedit Library the library was built against
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return "1.0.0.0"; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return Name;
        }
        /// <summary>
        /// Gets the tool tip to be displayed in the tool bar
        /// </summary>
        /// <value></value>
        public string MenuToolTip
        {
            get { return "Stamps a randomly chosen tile from the selected brush onto the layer"; }
        }
    
        /// <summary>
        /// Gets the icon to be used in the tool bar.
        /// </summary>
        /// <value>The icon.</value>
        public Bitmap Icon
        {
            get { return new Bitmap(Editor.GetPathFromRelative("Brushes\\RAndomStamper\\stamp_random.gif")); }
        }


    }
}
