using System;
using System.Collections.Generic;
using System.Text;
using PernEdit.Plugin;
using PernEdit.Properties;
using System.Drawing;
using PernEdit.GraphicsHelpers;
using PernEdit.WorldEntities;
using PernEdit;


namespace Default
{
    /// <summary>
    /// A rubber stamper brush stamper
    /// </summary>
    public class BrushDefaultStamper : IBrushStamperPlugin
    {

        /// <summary>
        /// Called when the Stamp operation iscomplete.
        /// </summary>
        /// <param name="x">The final x location.</param>
        /// <param name="y">The final y location.</param>
        public void Stamp(TileBrush brush, Layer layer, Rectangle area)
        {
            int dx = area.Width;
            int dy = area.Height;
            
            //draw to a temp layer instead of directly onto the target layer as that would bypass the undo stack
            Layer temp = new Layer("temp", dx, dy, brush.BrushLayer.TileWidth, brush.BrushLayer.TileHeight);

            for (int iy = 0; iy < dy; iy += brush.BrushLayer.Height)
            {
                for (int ix = 0; ix < dx; ix += brush.BrushLayer.Width)
                {
                    temp.Stamp(brush.BrushLayer, ix, iy, true);
                }
            }

            layer.Stamp(temp, area.X, area.Y, true);

        }

        /// <summary>
        /// Gets the bitmap.
        /// </summary>
        /// <param name="brush">The brush.</param>
        /// <returns></returns>
        public Bitmap GetBitmap(TileBrush brush, Rectangle area)
        {
          return brush.BrushLayer.GetBitmap();
        }
   
        /// <summary>
        /// Enumerated type of the plugin
        /// </summary>
        /// <value></value>
        public PluginType Type
        {
            get { return PluginType.PLUGIN_TYPE_BRUSH; }
        }

        /// <summary>
        /// The name of the plugin.  Will be used by the app to list the plugin
        /// </summary>
        /// <value></value>
        public string Name
        {
            get { return "Rubber Stamper"; }
        }

        /// <summary>
        /// GUID unique identifier for the plugin to avoid name clashes
        /// Create a new GUID for each brush plugin you create
        /// </summary>
        /// <value></value>
        public Guid ID
        {
            get { return new Guid("B3CAFE11-66E6-4c3b-B4B9-5F31E3E21A17"); }
        }

        /// <summary>
        /// Version of the Pernedit Library the library was built against
        /// </summary>
        /// <value></value>
        public string Version
        {
            get { return "1.0.0.0"; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return Name;
        }

        /// <summary>
        /// Gets the icon to be used in the tool bar.
        /// </summary>
        /// <value>The icon.</value>
        public System.Drawing.Bitmap Icon
        {
            get { return new Bitmap(Editor.GetPathFromRelative("Brushes\\RubberStamper\\stamp_basic.gif")); }
        }

        /// <summary>
        /// Gets the tool tip to be displayed in the tool bar
        /// </summary>
        /// <value></value>
        public string MenuToolTip
        {
            get { return "Standard brush stamper"; }
        } 
    }
}
